/*
 * Operating system kernel abstraction -- memory mapped I/O.
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#ifndef __OSKA_IO_H
#define __OSKA_IO_H

#ifdef __OSKA_API_DOC
/**
 * @defgroup io Memory mapped I/O.
 *
 * A set of functions are provided for accessing memory mapped I/O
 * registered of various widths (8, 16, 32 and 64 bit wide registers).
 */

/**
 * The handle to the start of a memory mapped I/O region.
 *
 * @ingroup io
 */
typedef implementation_defined os_io_mem_t;

uint8_t os_io_read8(os_io_mem_t base, unsigned offset);
uint16_t os_io_read16(os_io_mem_t base, unsigned offset);
uint32_t os_io_read32(os_io_mem_t base, unsigned offset);
uint64_t os_io_read64(os_io_mem_t base, unsigned offset);

void os_io_write8(os_io_mem_t base, unsigned offset, uint8_t val);
void os_io_write16(os_io_mem_t base, unsigned offset, uint16_t val);
void os_io_write32(os_io_mem_t base, unsigned offset, uint32_t val);
void os_io_write64(os_io_mem_t base, unsigned offset, uint64_t val);

/**
 * A memory barrier.
 *
 * This prevents the compilier and CPU from reordering memory accesses
 * across the barrier.  i.e., all loads and stores before the barrier
 * are completed before any loads and stores after the barrier.
 *
 * @ingroup io
 */
void os_io_memory_barrier(void);

#endif /* __OSKA_API_DOC */

#ifdef linux
#  include <../linux/io.h>
#elif defined(KMDF_MAJOR_VERSION)
#  include <../windows/kmdf/io.h>
#else
#  error <oska/io.h> not provided for this OS
#endif

#endif /* __OSKA_IO_H */
