/*
 * Operating system kernel abstraction -- standard types
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#ifndef __OSKA_TYPES_H
#define __OSKA_TYPES_H

#ifdef __OSKA_API_DOC
/**
 * @defgroup types Standard types
 *
 * <oska/types.h> makes available the standard (ISO C) integral
 * fixed-width types (\e uint8_t, \e uint16_t, \e uint32_t, \e int8_t,
 * \e int16_t, \e int32_t) and \e bool (plus the \e true and \e false
 * values).  \e int64_t and \e uint64_t are provided if supported by
 * the platform.
 *
 * @note On some implementation, \e bool is not the ISO C bool type
 * but is an integral type.  Therefore, care must be take to not
 * compare \e bool variables with \e true or \e false.
 */

#endif /* __OSKA_API_DOC */

#ifdef linux
#  include <../linux/types.h>
#elif defined(ZSDE)
#  include <../zoran/types.h>
#elif defined(KMDF_MAJOR_VERSION)
#  include <../windows/kmdf/types.h>
#else
#  error <oska/types.h> not provided for this OS
#endif

#endif /* #ifndef __OSKA_TYPES_H */
