/*
 * OSKA Posix implementation -- inter-thread message queues.
 *
 * Copyright (C) 2010 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#ifndef __OSKA_MQ_H
#define __OSKA_MQ_H

#include <oska/types.h>

#define OS_MQ_POSIX_READ_FD 0
#define OS_MQ_POSIX_WRITE_FD 1

typedef struct os_mq_posix {
    int fds[2];
    size_t mesg_len;
} os_mq_t;

typedef int os_mq_event_handle_t;

int os_mq_init(os_mq_t *mq, size_t message_len);
void os_mq_destroy(os_mq_t *mq);

int os_mq_dequeue(os_mq_t *mq, void *message);
int os_mq_enqueue(os_mq_t *mq, void *message);

/**
 * The handle of an event object for the message queue.
 *
 * This returns the handle to an OS-specific object that can be used
 * in an application to wake-up an OS-specific event loop when a
 * message is available on the queue.
 *
 * <h2>POSIX</h2>
 *
 * On POSIX systems this is a read file descriptor that can be added
 * to a select(2) or poll(2) system call.
 *
 * @param mq The message queue.
 *
 * @return the read file descriptor.
 */
static inline os_mq_event_handle_t os_mq_event_handle(os_mq_t *mq)
{
    return mq->fds[OS_MQ_POSIX_READ_FD];
}

#endif
