/*
 * OSKA Posix implementation -- spinlocks
 *
 * Copyright (C) 2010 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#ifndef __OSKA_POSIX_SPINLOCK_H
#define __OSKA_POSIX_SPINLOCK_H

#include <pthread.h>

typedef pthread_spinlock_t os_spinlock_t;
typedef int os_int_status_t;

static inline void os_spinlock_init(os_spinlock_t *lock)
{
    pthread_spin_init(lock, 0);
}

static inline void os_spinlock_destroy(os_spinlock_t *lock)
{
    pthread_spin_destroy(lock);
}

static inline void os_spinlock_lock_intsave(os_spinlock_t *lock,
					    os_int_status_t *int_state)
{
    pthread_spin_lock(lock);
}

static inline void os_spinlock_unlock_intrestore(os_spinlock_t *lock,
						 os_int_status_t *int_state)
{
    pthread_spin_unlock(lock);
}

#endif /* #ifndef __OSKA_POSIX_SPINLOCK_H */
