/*
 * OSKA Posix implementation -- tracing messages.
 *
 * Copyright (C) 2010 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#ifndef __OSKA_POSIX_TRACE_H
#define __OSKA_POSIX_TRACE_H

#ifndef OS_TRACE_PREFIX
#  define OS_TRACE_PREFIX ""
#endif

#ifdef OS_TRACE_USE_XORG

void os_trace_xorg_err(const char *format, ...);
void os_trace_xorg_warn(const char *format, ...);
void os_trace_xorg_info(const char *format, ...);
void os_trace_xorg_dbg(const char *format, ...);

#define os_trace_err(format, ...)  os_trace_xorg_err(format "\n", ## __VA_ARGS__)
#define os_trace_warn(format, ...) os_trace_xorg_warn(format "\n", ## __VA_ARGS__)
#define os_trace_info(format, ...) os_trace_xorg_info(format "\n", ## __VA_ARGS__)
#define os_trace_dbg(format, ...)  os_trace_xorg_dbg(format "\n", ## __VA_ARGS__)

#else

void os_trace_stdio(const char *format, ...);

#define os_trace_err(format, ...)  os_trace_stdio("E: " format "\n", ## __VA_ARGS__)
#define os_trace_warn(format, ...) os_trace_stdio("W: " format "\n", ## __VA_ARGS__)
#define os_trace_info(format, ...) os_trace_stdio("I: " format "\n", ## __VA_ARGS__)
#define os_trace_dbg(format, ...)  os_trace_stdio("D: " format "\n", ## __VA_ARGS__)

#endif

#endif /* #ifndef __OSKA_POSIX_TRACE_H */
