/*
 * OSKA Windows KMDF implementation -- memory allocation.
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#pragma once
#include <ntddk.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Use the same tag for all allocations.
 */
#define OSKA_POOL_TAG 'akso'

__inline void *os_alloc_nonzeroed(size_t size)
{
    return ExAllocatePoolWithTag(NonPagedPool, size, OSKA_POOL_TAG);
}

__inline void *os_alloc(size_t size)
{
    void *ptr;

    ptr = os_alloc_nonzeroed(size);
    if (ptr) {
        memset(ptr, 0, size);
    }
    return ptr;
}

__inline void os_free(void *ptr)
{
    if (ptr) {
        ExFreePoolWithTag(ptr, OSKA_POOL_TAG);
    }
}

void *os_alloc_big(size_t size);
void os_free_big(void *ptr);

#ifdef __cplusplus
} /* extern "C" */
#endif
