/*
 * OSKA Windows KMDF implementation -- events.
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#include <oska/event.h>

void os_event_init(os_event_t *evt)
{
    KeInitializeEvent(&evt->event, NotificationEvent, FALSE);
    KeInitializeSpinLock(&evt->lock);
    evt->events = 0;
}

uint16_t os_event_wait(os_event_t *evt)
{
    uint16_t e;
    KIRQL irql;

    KeWaitForSingleObject(&evt->event, Executive, KernelMode, FALSE, NULL);

    KeAcquireSpinLock(&evt->lock, &irql);
    e = evt->events;
    evt->events &= ~e;
    KeClearEvent(&evt->event);
    KeReleaseSpinLock(&evt->lock, irql);

    return e;
}

uint16_t os_event_wait_timed(os_event_t *evt, unsigned timeout_ms)
{
    uint16_t e;
    LARGE_INTEGER to;
    KIRQL irql;

    to.QuadPart = -(timeout_ms * 10000ll);
    KeWaitForSingleObject(&evt->event, Executive, KernelMode, FALSE, &to);

    KeAcquireSpinLock(&evt->lock, &irql);
    e = evt->events;
    evt->events &= ~e;
    KeClearEvent(&evt->event);
    KeReleaseSpinLock(&evt->lock, irql);

    return e;
}

void os_event_raise(os_event_t *evt, uint16_t events)
{
    KIRQL irql;
    
    KeAcquireSpinLock(&evt->lock, &irql);
    evt->events |= events;
    KeReleaseSpinLock(&evt->lock, irql);

    KeSetEvent(&evt->event, 0, FALSE);
}
