/*
 * OSKA Windows KMDF implementation -- memory mapped I/O.
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#pragma once
#include <oska/types.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef void *os_io_mem_t;

__inline uint8_t os_io_read8(os_io_mem_t base, unsigned offset)
{
    return READ_REGISTER_UCHAR((void *)((uint8_t *)base + offset));
}

__inline uint16_t os_io_read16(os_io_mem_t base, unsigned offset)
{
    return READ_REGISTER_USHORT((void *)((uint8_t *)base + offset));
}

__inline uint32_t os_io_read32(os_io_mem_t base, unsigned offset)
{
    return READ_REGISTER_ULONG((void *)((uint8_t *)base + offset));
}

__inline uint64_t os_io_read64(os_io_mem_t base, unsigned offset)
{
#if defined(_WIN64)
    return READ_REGISTER_ULONG64((void *)((uint8_t *)base + offset));
#else
    uint64_t vl, vh;
    vl = READ_REGISTER_ULONG((void *)((uint8_t *)base + offset));
    vh = READ_REGISTER_ULONG((void *)((uint8_t *)base + offset + 4));
    return (vh << 32) | vl;
#endif
}

__inline void os_io_write8(os_io_mem_t base, unsigned offset, uint8_t val)
{
    WRITE_REGISTER_UCHAR((void *)((uint8_t *)base + offset), val);
}

__inline void os_io_write16(os_io_mem_t base, unsigned offset, uint16_t val)
{
    WRITE_REGISTER_USHORT((void *)((uint8_t *)base + offset), val);
}

__inline void os_io_write32(os_io_mem_t base, unsigned offset, uint32_t val)
{
    WRITE_REGISTER_ULONG((void *)((uint8_t *)base + offset), val);
}

__inline void os_io_write64(os_io_mem_t base, unsigned offset, uint64_t val)
{
#if defined(_WIN64)
    WRITE_REGISTER_ULONG64((void *)((uint8_t *)base + offset), val);
#else
    WRITE_REGISTER_ULONG((void *)((uint8_t *)base + offset),     (val >>  0) & 0xffffffff);
    WRITE_REGISTER_ULONG((void *)((uint8_t *)base + offset + 4), (val >> 32) & 0xffffffff);
#endif
}

__inline void os_io_memory_barrier(void)
{
    KeMemoryBarrier();
}

#ifdef __cplusplus
} /* extern "C" */
#endif
