/*
 * OSKA Windows KMDF implementation -- DLL entry/exit points.
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#include <oska/all.h>
#include <stdarg.h>
#include <ntstrsafe.h>

/**
 * Map an NTSTATUS value to a negative error code.
 *
 * All success or information statuses are mapped to 0.
 *
 * Error codes are returned as-is as they're already negative 32 bit
 * values.  This allows errors from OS calls to be passed unchanged to
 * the OS-specific top edge of drivers through the OS-independant
 * middle layer of drivers.
 */
int os_ntstatus_to_err(NTSTATUS status)
{
    if (NT_SUCCESS(status)) {
        return 0;
    }
    return (INT32)status;
}

NTSTATUS os_err_to_ntstatus(int err)
{
    return (NTSTATUS)err;
}

/*
 * An implementation of the C99 function snprintf().
 *
 * It does not conform fully to C99, if the buffer would overflow
 * 'size' is returned instead of the length had there been enough
 * space.
 */
int snprintf(char *str, size_t size, const char *format, ...)
{
    va_list args;
    size_t remaining_space;
    NTSTATUS status;

    va_start(args, format);
    status = RtlStringCbVPrintfExA(str, size, NULL, &remaining_space, 0, format, args);
    va_end(format);

    if (status == STATUS_BUFFER_OVERFLOW) {
        return size;
    }
    return size - remaining_space;
}
