/*
 * Windows KMDF event functions.
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#pragma once
#include <ntddk.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef KSPIN_LOCK os_spinlock_t;
typedef KIRQL os_int_status_t;

__inline void os_spinlock_init(os_spinlock_t *lock)
{
    KeInitializeSpinLock(lock);
}

__inline void os_spinlock_destroy(os_spinlock_t *lock)
{
    /* no op */
}

__drv_raisesIRQL(DISPATCH_LEVEL)
__inline void os_spinlock_lock_intsave(
    os_spinlock_t *lock,
    __drv_deref(__drv_savesIRQL) os_int_status_t *int_state)
{
    KeAcquireSpinLock(lock, int_state);
}

__drv_requiresIRQL(DISPATCH_LEVEL)
__inline void os_spinlock_unlock_intrestore(
    os_spinlock_t *lock,
    __drv_deref(__drv_restoresIRQL) os_int_status_t *int_state)
{
    KeReleaseSpinLock(lock, *int_state);
}

#ifdef __cplusplus
} /* extern "C" */
#endif
