/*
 * OSKA Windows KMDF implementation -- threads.
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#include <oska/thread.h>

static KSTART_ROUTINE thread_func;

int os_thread_create(os_thread_t *thread, const char *name,
                     void (*func)(void *), void *arg)
{
    HANDLE handle;
    OBJECT_ATTRIBUTES attrs;
    NTSTATUS status;

    thread->func = func;
    thread->arg = arg;
    thread->stop = 0;

    InitializeObjectAttributes(&attrs, NULL, OBJ_KERNEL_HANDLE, NULL, NULL);
    status = PsCreateSystemThread(&handle, GENERIC_ALL, &attrs, NULL, NULL,
                                  thread_func, thread);
    if (NT_SUCCESS(status)) {
        ObReferenceObjectByHandle(handle, THREAD_ALL_ACCESS, NULL, KernelMode,
                                  &thread->thread, NULL);
        ZwClose(handle);
    }
    return os_ntstatus_to_err(status);
}

void os_thread_stop(os_thread_t *thread, os_event_t *event)
{
    thread->stop = 1;
    if (event) {
        os_event_raise(event, ~0);
    }
    KeWaitForSingleObject(thread->thread, Executive, KernelMode, FALSE, NULL);
    ObDereferenceObject(thread->thread);
}

int os_thread_should_stop(os_thread_t *thread)
{
    return thread->stop;
}

static VOID thread_func(PVOID data)
{
    os_thread_t *thread = data;

    thread->func(thread->arg);

    PsTerminateSystemThread(0);
}
