/*
 * OSKA Windows KMDF implementation -- timers.
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#include <oska/timer.h>

static KDEFERRED_ROUTINE timer_func;

void os_timer_init(os_timer_t *timer, os_timer_func_t func, void *arg)
{
    timer->func = func;
    timer->arg = arg;
    KeInitializeTimer(&timer->timer);
    KeInitializeDpc(&timer->dpc, timer_func, timer);
}

void os_timer_destroy(os_timer_t *timer)
{
    KeCancelTimer(&timer->timer);
    KeFlushQueuedDpcs();
}

void os_timer_set(os_timer_t *timer, unsigned long expires_ms)
{
    LARGE_INTEGER t;

    t.QuadPart = -(expires_ms * 10000ll);
    KeSetTimer(&timer->timer, t, &timer->dpc);
}

static void timer_func(KDPC *dpc, void *context, void *arg1, void *arg2)
{
    os_timer_t *timer = context;

    timer->func(timer->arg);
}
