/*
 * Zoran console printing functions.
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#include <oska/print.h>

#include <stdio.h>
#include <stdarg.h>

#include "Debug.h"

void os_print(enum os_print_level level, const char *prefix, const char *name,
              const char *format, ...)
{
    va_list va_args;

    va_start(va_args, format);
    os_vprint(level, prefix, name, format, va_args);
    va_end(va_args);
}

void os_vprint(enum os_print_level level, const char *prefix, const char *name,
               const char *format, va_list args)
{
    const char *level_str[] = {
        [OS_PRINT_ERROR]   = ZORAN_ERROR_PREFIX,
        [OS_PRINT_WARNING] = ZORAN_WARNING_PREFIX,
        [OS_PRINT_INFO]    = ZORAN_INFO_PREFIX,
        [OS_PRINT_DEBUG]   = ZORAN_DEBUG_PREFIX,
    };
    char buf[80];
    int w = 0;

    if (name) {
        w += snprintf(buf + w, sizeof(buf) - w, "%s%s%s: ", level_str[level], prefix, name);
    } else {
        w += snprintf(buf + w, sizeof(buf) - w, "%s%s", level_str[level], prefix);
    }
    w += vsnprintf(buf + w, sizeof(buf) - w, format, args);
    Printf_B("%s\n", buf);
}
