/*
 * OSKA Zoran HCE implementation -- spinlocks
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#ifndef __OSKA_ZORAN_SPINLOCK_H
#define __OSKA_ZORAN_SPINLOCK_H

#include "Os.h"

typedef TX_MUTEX os_spinlock_t;
typedef unsigned long os_int_status_t;

/* os_spinlock_init() create options */
#define __OSKA_ZORAN_SPINLOCK_CREATE_OPTIONS	TX_NO_INHERIT
/* os_spinlock_lock_intsave() wait option */
#define __OSKA_ZORAN_SPINLOCK_GET_WAIT	TX_WAIT_FOREVER

static inline void os_spinlock_init(os_spinlock_t *lock)
{
	OsMutexCreate(lock, "oska_spinlock", __OSKA_ZORAN_SPINLOCK_CREATE_OPTIONS);
}

static inline void os_spinlock_destroy(os_spinlock_t *lock)
{
	OsMutexDelete(lock);
}

static inline void os_spinlock_lock_intsave(os_spinlock_t *lock,
					    os_int_status_t *int_state)
{
	(void)int_state;
	tx_interrupt_control(TX_INT_DISABLE);
	OsMutexGet(lock, __OSKA_ZORAN_SPINLOCK_GET_WAIT);
}

static inline void os_spinlock_unlock_intrestore(os_spinlock_t *lock,
						 os_int_status_t *int_state)
{
	(void)int_state;
	OsMutexPut(lock);
	tx_interrupt_control(TX_INT_ENABLE);
}

#endif /* #ifndef __OSKA_ZORAN_SPINLOCK_H */
