/*
 * Zoran HCE thread functions.
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#include <oska/thread.h>

static VOID thread_func(ULONG data)
{
    os_thread_t *thread = (os_thread_t *)data;

    thread->func(thread->arg);
}

int os_thread_create(os_thread_t *thread, const char *name, void (*func)(void *), void *arg)
{
    thread->func = func;
    thread->arg = arg;

    thread->stop = 0;

    if(NULL == (thread->task = OsThreadCreateEx((CHAR*)name, thread_func, (ULONG)thread,
    		1, 1, 3, TX_AUTO_START)))
    	return -1;

    return 0;
}

void os_thread_stop(os_thread_t *thread, os_semaphore_t *sem)
{
    /*
     * Stop flag must be set before the semaphore is posted so
     * kthread_should_stop() cannot be used.
     */
    thread->stop = 1;

    if (sem) {
        os_semaphore_post(sem);
    }

    OsThreadTerminate(thread->task);
}

int os_thread_should_stop(os_thread_t *thread)
{
    return thread->stop;
}
