/*
 * Synergy compatible API (additional library functions).
 *
 * Copyright (C) 2010 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#include "csr_sdio_internal.h"

CsrResult CsrSdioFunctionReenable(CsrSdioFunction *function)
{
    struct glue_func *glue = function->priv;
    CsrStatus ret;

    ret = sdioemb_reenable_csr_function(glue->fdev);
    return CsrSdioStatusToResult(ret);
}

CsrStatus CsrSdioResultToStatus(CsrResult result)
{
    switch (result) {
    case CSR_RESULT_SUCCESS:
        return 0;
    case CSR_RESULT_FAILURE:
        return -ENOMEM;
    case CSR_SDIO_RESULT_INVALID_VALUE:
        return -EINVAL;
    case CSR_SDIO_RESULT_NO_DEVICE:
        return -ENODEV;
    case CSR_SDIO_RESULT_CRC_ERROR:
        return -EIO;
    case CSR_SDIO_RESULT_TIMEOUT:
        return -ETIMEDOUT;
    case CSR_SDIO_RESULT_NOT_RESET:
        return 0;
    default:
        return -EIO; /* arbitrary */
    }
}

CsrResult CsrSdioStatusToResult(CsrStatus status)
{
    if (!CSR_STATUS_FAILURE(status)) {
        return CSR_RESULT_SUCCESS;
    }

    switch (status) {
    case -EIO:
        return CSR_SDIO_RESULT_CRC_ERROR;
    case -ETIMEDOUT:
        return CSR_SDIO_RESULT_TIMEOUT;
    case -ENODEV:
    case -ENOMEDIUM:
        return CSR_SDIO_RESULT_NO_DEVICE;
    case -ENOMEM:
        return CSR_RESULT_FAILURE; /* There is no CSR_RESULT_NO_MEMORY. */
    case -EINVAL:
        return CSR_SDIO_RESULT_INVALID_VALUE;
    default:
        return CSR_RESULT_FAILURE;
    }
}
