/*
 * Compile time configuration defaults.
 *
 * Copyright (C) 2007 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#ifndef _SDIOEMB_CONFIG_H
#define _SDIOEMB_CONFIG_H

/**
 * @defgroup config Compile-time configuration
 *
 * Various compile time options are set with \#define's.  These may be
 * overridden by providing appropriate compiler options (e.g., with
 * gcc's -D option).
 *
 * Defaults are provided in sdio_config.h.
 *
 *@{
 */

/**
 * @def SDIOEMB_SLOTS_MAX
 * 
 * Maximum number of SDIO slots that may be registered with the core.
 */ 
#ifndef SDIOEMB_SLOTS_MAX
#  define SDIOEMB_SLOTS_MAX 4
#endif

/**
 * @def SDIOEMB_FDRIVERS_MAX
 *
 * Maxium number of function drivers that may be registered with the
 * core.
 */
#ifndef SDIOEMB_FDRIVERS_MAX
#  define SDIOEMB_FDRIVERS_MAX 4
#endif

/**
 * @def SDIOEMB_FDEVS_MAX
 *
 * Maximum number of functions supported by the core.
 */
#ifndef SDIOEMB_FDEVS_MAX
#  define SDIOEMB_FDEVS_MAX 8
#endif

/**
 * @def SDIOEMB_CARD_INIT_RETRY_MAX
 *
 * Maximum number of times to try to initialize a card.
 */
#ifndef SDIOEMB_CARD_INIT_RETRY_MAX
#  define SDIOEMB_CARD_INIT_RETRY_MAX 3
#endif

/**
 * @def SDIOEMB_CARD_READY_TIMEOUT_MS
 *
 * Time (in ms) to wait for the card to become ready during the
 * initialization sequence.
 */
#ifndef SDIOEMB_CARD_READY_TIMEOUT_MS
#  define SDIOEMB_CARD_READY_TIMEOUT_MS 100
#endif

/**
 * @def SDIOEMB_CARD_IS_REMOVABLE
 *
 * Non-zero if the card is removable, zero if the card is always
 * inserted (e.g., it's wired directly to the SD host controller).
 *
 * A thread is used when detecting removable cards.
 */
#ifndef SDIOEMB_CARD_IS_REMOVABLE
#  define SDIOEMB_CARD_IS_REMOVABLE 1
#endif

/**
 * @def SDIOEMB_CSPI_REG_PADDING
 *
 * Amount of CSPI register padding in octets. The default of 0 should
 * be fine for all chips and bus speeds less than 25 MHz.
 */
#ifndef SDIOEMB_CSPI_REG_PADDING
#  define SDIOEMB_CSPI_REG_PADDING 1
#endif

/**
 * @def SDIOEMB_CSPI_BURST_PADDING
 *
 * Amount of CSPI burst padding in octets. The default of 2 should be
 * fine for all chips and bus speeds less than 25 MHz.
 */
#ifndef SDIOEMB_CSPI_BURST_PADDING
#  define SDIOEMB_CSPI_BURST_PADDING 2
#endif

/*@}*/

#endif /* _SDIOEMB_CONFIG_H */
