/*
 * API for SDIO function drivers on Windows.
 *
 * Copyright (C) 2010 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#pragma once
#include <sdioemb/sdio_api.h>

/**
 * The driver interface to obtain sdioemb's device.
 *
 * The GUID for this interface is defined as
 * GUID_DRVINTERFACE_SDIOEMB_FUNC.
 *
 * @ingroup windows
 */
struct sdioemb_wdf_interface {
    INTERFACE header;
    struct sdioemb_dev *fdev;

    int (*driver_probe)(struct sdioemb_func_driver *fdriver, struct sdioemb_dev *fdev);
    void (*driver_remove)(struct sdioemb_func_driver *fdriver, struct sdioemb_dev *fdev);

    int (*set_block_size)(struct sdioemb_dev *fdev, uint16_t blksz);
    void (*set_max_bus_freq)(struct sdioemb_dev *fdev, int max_freq);

    int (*enable_function)(struct sdioemb_dev *fdev);
    int (*disable_function)(struct sdioemb_dev *fdev);
    int (*reenable_csr_function)(struct sdioemb_dev *fdev);
    void (*idle_function)(struct sdioemb_dev *fdev);

    int (*read8)(struct sdioemb_dev *fdev, uint32_t addr, uint8_t *val);
    int (*read16)(struct sdioemb_dev *fdev, uint32_t addr, uint16_t *val);
    int (*write8)(struct sdioemb_dev *fdev, uint32_t addr, uint8_t val);
    int (*write16)(struct sdioemb_dev *fdev, uint32_t addr, uint16_t val);
    int (*f0_read8)(struct sdioemb_dev *fdev, uint32_t addr, uint8_t *val);
    int (*f0_write8)(struct sdioemb_dev *fdev, uint32_t addr, uint8_t val);
    int (*read)(struct sdioemb_dev *fdev, uint32_t addr, void *data, size_t len);
    int (*write)(struct sdioemb_dev *fdev, uint32_t addr, const void *data, size_t len);

    int (*hard_reset)(struct sdioemb_dev *fdev);

    void (*power_on)(struct sdioemb_dev *fdev);
    void (*power_off)(struct sdioemb_dev *fdev);

    int (*interrupt_enable)(struct sdioemb_dev *fdev);
    int (*interrupt_disable)(struct sdioemb_dev *fdev);
    void (*interrupt_acknowledge)(struct sdioemb_dev *fdev);

    void (*suspend_function)(struct sdioemb_dev *fdev);
    void (*resume_function)(struct sdioemb_dev *fdev);
};
