/*
 * Driver declarations and definitions.
 *
 * Copyright (C) 2007 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#ifndef __BT_A_DEV_H
#define __BT_A_DEV_H

#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/workqueue.h>
#include <linux/poll.h>
#include <linux/device.h>
#include <linux/delay.h>
#include <linux/slab.h>

#include <asm/uaccess.h>

#include <sdioemb/sdio.h>
#include <sdioemb/sdio_csr.h>
#include <sdioemb/sdio_cis.h>
#include <sdioemb/sdio_bt_a.h>
#include <csr_sdio.h>
#include <csr_sdio_lib.h>

#define BC_DEEP_SLEEP_EXIT_TIMEOUT_MS 50

struct rx_queue_entry {
    struct list_head node;
    size_t len;
    size_t read;
    uint8_t *data;
};

struct bt_a_device {
    struct sdioemb_bt_a_dev     bdev;
    CsrSdioFunction *           func;

    int                         minor;
    int                         card_present;
    struct mutex                mutex;
    struct workqueue_struct *   workqueue;
    struct work_struct          irq_work;
    wait_queue_head_t           read_waitq;
    spinlock_t                  rx_queue_lock;
    struct list_head            rx_queue;
    int                         rx_queue_len;
#define RX_QUEUE_LEN_MAX 20
    wait_queue_head_t           drowsy_waitq;
    struct timer_list           idle_timer;
    struct work_struct          torpid_work;
    struct cdev                 cdev;
    atomic_t                    ref_count;
};

#define bt_err(bt, fmt, args...) printk(KERN_ERR "sdio_bt_a%d: " fmt, \
                                       (bt)->fdev->slot_id, ## args)
#define bt_warn(bt, fmt, args...) printk(KERN_WARNING "sdio_bt_a%d: " fmt, \
                                        (bt)->fdev->slot_id, ## args)

int card_driver_register(void);
void card_driver_unregister(void);

int needs_read_ack(struct bt_a_device *bt);
void start_function(struct bt_a_device *bt);
int check_for_reset(struct bt_a_device *bt);
int tx_packet(struct bt_a_device *bt, const char *packet, ssize_t len);
void set_sleep_state(struct bt_a_device *bt, enum sdio_sleep_state state);
void handle_interrupt(struct bt_a_device *bt);

void fix_quirks(struct bt_a_device *bt);

void rx_queue_add(struct bt_a_device *bt, struct rx_queue_entry *rx_entry);
struct rx_queue_entry *rx_queue_head(struct bt_a_device *bt);
void rx_queue_remove(struct bt_a_device *bt, struct rx_queue_entry *rx_entry);

#endif /* #ifndef __BT_A_DEV_H */
