/*
 * Receive packet queue operations.
 *
 * Copyright (C) 2007 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#include "bt_a_dev.h"

/**
 * Add an entry to the tail of the receive queue.
 *
 * @param bt       the BT device.
 * @param rx_entry entry to add.
 */
void rx_queue_add(struct bt_a_device *bt, struct rx_queue_entry *rx_entry)
{
    spin_lock(&bt->rx_queue_lock);

    list_add_tail(&rx_entry->node, &bt->rx_queue);
    bt->rx_queue_len++;

    spin_unlock(&bt->rx_queue_lock);

    wake_up_interruptible(&bt->read_waitq);

    if (bt->rx_queue_len >= RX_QUEUE_LEN_MAX) {
        sdioemb_bt_a_rx_off(&bt->bdev);
    }
}

/**
 * Return the first entry in the queue (or NULL if the queue is
 * empty).
 *
 * @param bt the BT device.
 */
struct rx_queue_entry *rx_queue_head(struct bt_a_device *bt)
{
    struct rx_queue_entry *rx_entry = NULL;

    spin_lock(&bt->rx_queue_lock);
    if (!list_empty(&bt->rx_queue)) {
        rx_entry = list_entry(bt->rx_queue.next, struct rx_queue_entry, node);
    }
    spin_unlock(&bt->rx_queue_lock);

    return rx_entry;
}

/**
 * Remove an entry from the receive queue.
 *
 * @param bt       the BT device.
 * @param rx_entry the entry to remove.
 */
void rx_queue_remove(struct bt_a_device *bt, struct rx_queue_entry *rx_entry)
{
    spin_lock(&bt->rx_queue_lock);

    list_del(&rx_entry->node);
    bt->rx_queue_len--;

    spin_unlock(&bt->rx_queue_lock);

    if (bt->rx_queue_len <= RX_QUEUE_LEN_MAX/2) {
        sdioemb_bt_a_rx_on(&bt->bdev);
    }
}
