/*
 * libsdio internal header.
 *
 * Copyright (C) 2010 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#ifndef LIBSDIO_INTERNAL_H
#define LIBSDIO_INTERNAL_H

#include <windows.h>
#include <winioctl.h>

typedef UINT32 uint32_t;
typedef INT32 int32_t;
typedef UINT16 uint16_t;
typedef UINT8 uint8_t;

#include <sdioemb/windows/uif.h>
#include <sdioemb/sdio.h>
#include <sdioemb/libsdio.h>

struct cmd_queue_elem {
    struct cmd_queue_elem *next;
    sdio_io_callback_t callback;
    void *arg;
    struct sdioemb_uif_cmd cmd;
    uint8_t write8_data;
};

struct cmd_queue {
    struct cmd_queue_elem *head;
    struct cmd_queue_elem *tail;
};

struct sdio_uif {
    void (*int_handler)(struct sdio_uif *, void *);
    void *arg;

    HANDLE handle;
    HANDLE int_thread;
    HANDLE int_thread_stop_event;
    HANDLE int_unmasked_event;

    struct sdioemb_uif_card_info card_info;
    struct sdioemb_uif_func_info func_info[SDIO_MAX_FUNCTIONS];

    struct cmd_queue cmd_queue;
    HANDLE cmd_queue_sem;
    HANDLE cmd_queue_stop_event;
    CRITICAL_SECTION cmd_queue_mutex;
    HANDLE async_thread;
};

int uif_ioctl(sdio_uif_t uif, DWORD ioctl, struct sdioemb_uif_input *input,
              struct sdioemb_uif_output *output);

int async_thread_start(sdio_uif_t uif);
void async_thread_stop(sdio_uif_t uif);

#endif /* #ifndef LIBSDIO_INTERNAL_H */
