/*
 * Sdioemb event log viewer application.
 *
 * Copyright (C) 2010 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#define _UNICODE
#define UNICODE
#include <tchar.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <windows.h>
#include <Strsafe.h>
#include <setupapi.h>

#include <initguid.h>
#include <sdioemb/windows/guid.h>

static TCHAR *find_iface(const GUID *guid, const TCHAR *suffix);
static BOOL path_matches(const TCHAR *path, const TCHAR *suffix);

INT __cdecl main(__in ULONG argc, __in_ecount(argc) PCHAR argv[])
{
    TCHAR suffix[32];
    TCHAR *device_path;
    UCHAR buffer[512]; /* enough for 6 lines of log */
    HANDLE log_file;
    int slot_number=0;
    DWORD Read = 0;

    if (argc > 1 && argv[1]) {
        slot_number = atoi(argv[1]);
    }

    StringCbPrintf(suffix, sizeof(suffix), TEXT("\\%d"), slot_number);

    device_path = find_iface(&GUID_DEVINTERFACE_SDIOEMB_EVENT_LOG, suffix);
    if (device_path == NULL) {
        fprintf(stderr, "E: slot number %d not found\n", slot_number);
        exit(1);
    }

    log_file = CreateFile(device_path,
                          GENERIC_READ, 
                          FILE_SHARE_READ, 
                          NULL, 
                          OPEN_EXISTING, 
                          FILE_ATTRIBUTE_NORMAL,
                          NULL);
    if (log_file == INVALID_HANDLE_VALUE) {
        fprintf(stderr, "E: could not open log file '%s' (%d)\n",
                device_path, GetLastError());
        exit(1);
    }

    for (;;) {
        ReadFile(log_file, buffer, sizeof(buffer), &Read, NULL);
        if (Read == 0) {
            break;
        }
        printf("%s", buffer);
    }
    CloseHandle(log_file);

    return 0;
}

/*
 * Find the device interface of type 'guid' whose path ends in
 * 'suffix'.
 */
static TCHAR *find_iface(const GUID *guid, const TCHAR *suffix)
{
    HDEVINFO                            devices;
    SP_DEVICE_INTERFACE_DATA            interface;
    PSP_DEVICE_INTERFACE_DETAIL_DATA    details = NULL, new_details;
    ULONG                               length = 0;
    int                                 i;
    TCHAR                              *device_path = NULL;

    devices = SetupDiGetClassDevs(guid, NULL, NULL, DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);
    if (devices == INVALID_HANDLE_VALUE) {
        return NULL;
    }

    interface.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);

    for (i = 0;; i++) {
        BOOL success;

        success = SetupDiEnumDeviceInterfaces(devices, 0, guid, i, &interface);
        if (!success) {
            if (GetLastError() == ERROR_NO_MORE_ITEMS) {
                break;
            }
            continue;
        }

        success = SetupDiGetDeviceInterfaceDetail(devices, &interface, NULL, 0, &length, NULL);
        if (!success && GetLastError() != ERROR_INSUFFICIENT_BUFFER) {
            continue;
        }

        free(details);
        details = malloc(length);
        if (details == NULL) {
            continue;
        }
        details->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

        success = SetupDiGetDeviceInterfaceDetail(
            devices, &interface, details, length, NULL, NULL);
        if (!success) {
            continue;
        }

        if (path_matches(details->DevicePath, suffix)) {
            device_path = _tcsdup(details->DevicePath);
            break;
        }
    }

    SetupDiDestroyDeviceInfoList(devices);
    free(details);
    return device_path;
}

static BOOL path_matches(const TCHAR *path, const TCHAR *suffix)
{
    size_t path_len, suffix_len;

    path_len = _tcslen(path);
    suffix_len = _tcslen(suffix);

    if (suffix_len > path_len) {
        return FALSE;
    }

    return _tcscmp(path + (path_len - suffix_len), suffix) == 0;
}
