/*
 * Sdioemb Windows KMDF SHC driver -- interrupt handling.
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#include "shc_internal.h"

static EVT_WDF_INTERRUPT_ISR shc_evt_interrupt_isr;
static EVT_WDF_INTERRUPT_DPC shc_evt_interrupt_dpc;

NTSTATUS shc_interrupt_init(WDFDEVICE device)
{
    struct shc_device_context *ctx = shc_get_device_context(device);
    WDF_INTERRUPT_CONFIG config;
    WDF_OBJECT_ATTRIBUTES attribs;
    NTSTATUS status;

    WDF_INTERRUPT_CONFIG_INIT(&config,
                              shc_evt_interrupt_isr,
                              shc_evt_interrupt_dpc);

    status = WdfInterruptCreate(device, &config, WDF_NO_OBJECT_ATTRIBUTES,
                                &ctx->interrupt);
    return status;
}


static BOOLEAN shc_evt_interrupt_isr(
    IN WDFINTERRUPT interrupt,
    IN ULONG message_id)
{
    WDFDEVICE device = WdfInterruptGetDevice(interrupt);
    struct shc_device_context *ctx = shc_get_device_context(device);
    uint32_t int_status;
    bool pending;
    int ints = 0;
    int i;

    for (i = 0; i < ctx->num_slots; i++) {
        struct sdioemb_slot *slot;
        struct shc_data *shc;

        slot = ctx->slots[i];
        shc =  slot->drv_data;

        pending = sdioemb_shc_isr(&shc->shc, &int_status);
        if (pending) {
            ints++;
            shc->int_status |= int_status;
        }
    }
    if (ints == 0) {
        return FALSE;
    }

    WdfInterruptQueueDpcForIsr(interrupt);

    return TRUE;
}

static VOID shc_evt_interrupt_dpc(
    IN WDFINTERRUPT interrupt,
    IN WDFDEVICE device)
{
    struct shc_device_context *ctx = shc_get_device_context(device);
    uint32_t int_status;
    int i;

    for (i = 0; i < ctx->num_slots; i++) {
        struct sdioemb_slot *slot;
        struct shc_data *shc;

        slot = ctx->slots[i];
        shc =  slot->drv_data;

        WdfInterruptAcquireLock(interrupt);
        int_status = shc->int_status;
        shc->int_status = 0;
        WdfInterruptReleaseLock(interrupt);

        sdioemb_shc_dsr(&shc->shc, int_status);
    }
}
