/*
 * Sdioemb Windows KMDF SHC driver -- PCI configuration space access.
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#include "shc_internal.h"

#include <wdmguid.h>

#include "pci_config.tmh"

NTSTATUS shc_pci_config_init(WDFDEVICE device)
{
    struct shc_device_context *dev_ctx = shc_get_device_context(device);
    NTSTATUS status;

    status = WdfFdoQueryForInterface(device, &GUID_BUS_INTERFACE_STANDARD,
                                     (void *)&dev_ctx->bus_interface,
                                     sizeof(dev_ctx->bus_interface), 1, NULL);
    if (!NT_SUCCESS(status)) {
        os_trace_err("can't get bus interface: %!STATUS!", status);
        return status;
    }
    return STATUS_SUCCESS;
}

void shc_pci_config_cleanup(WDFDEVICE device)
{
    struct shc_device_context *dev_ctx = shc_get_device_context(device);

    if (dev_ctx->bus_interface.InterfaceDereference) {
        dev_ctx->bus_interface.InterfaceDereference(dev_ctx->bus_interface.Context);
    }
}

void shc_pci_config_read8(WDFDEVICE device, uint32_t offset, uint8_t *val)
{
    struct shc_device_context *dev_ctx = shc_get_device_context(device);

    dev_ctx->bus_interface.GetBusData(dev_ctx->bus_interface.Context,
                                      PCI_WHICHSPACE_CONFIG, val, offset, sizeof(*val));
}

void shc_pci_config_write8(WDFDEVICE device, uint32_t offset, uint8_t val)
{
    struct shc_device_context *dev_ctx = shc_get_device_context(device);

    dev_ctx->bus_interface.SetBusData(dev_ctx->bus_interface.Context,
                                      PCI_WHICHSPACE_CONFIG, &val, offset, sizeof(val));
}
