/*
 * Sdioemb Windows KMDF SHC driver.
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#pragma once
#include <oska/all.h>
#include <sdioemb/slot_api.h>
#include <sdioemb/windows/slot_api.h>
#include <sdioemb/slot_shc.h>
#include <sdioemb/trace.h>

#define SHC_DMA_BUFFER_LEN 8192

typedef struct shc_device_context {
    const char *name;
    BUS_INTERFACE_STANDARD bus_interface;
    WDFINTERRUPT interrupt;
    WDFDMAENABLER dma_enabler;
    uint32_t quirks;
    int num_slots;
    struct sdioemb_slot **slots;
    bool registered;
} SHC_DEVICE_CONTEXT, *PSHC_DEVICE_CONTEXT;
WDF_DECLARE_CONTEXT_TYPE_WITH_NAME(SHC_DEVICE_CONTEXT, shc_get_device_context);

struct shc_data {
    struct sdioemb_wdf_slot_data wdf_data; /* must be first */
    struct sdioemb_shc shc;
    WDFDEVICE device;
    WDFCOMMONBUFFER dma_buf;
    void *dma_buf_virt;
    uint64_t dma_buf_phys;
    size_t addr_len;
    uint32_t int_status;
    bool use_dma;
};

NTSTATUS shc_registry_init(WDFDEVICE device);
void shc_registry_cleanup(WDFDEVICE device);

NTSTATUS shc_pci_config_init(WDFDEVICE device);
void shc_pci_config_cleanup(WDFDEVICE device);
void shc_pci_config_read8(WDFDEVICE device, uint32_t offset, uint8_t *val);
void shc_pci_config_write8(WDFDEVICE device, uint32_t offset, uint8_t val);

NTSTATUS shc_interrupt_init(WDFDEVICE device);
NTSTATUS shc_dma_init(WDFDEVICE device);

NTSTATUS shc_create_slot(WDFDEVICE device, struct sdioemb_slot **new_slot, int slot_idx);
void shc_destroy_slot(WDFDEVICE device, struct sdioemb_slot *slot);
