/*
 * Sdioemb Windows KMDF SHC driver.
 *
 * Copyright (C) 2009 Cambridge Silicon Radio Ltd.
 *
 * Refer to LICENSE.txt included with this source code for details on
 * the license terms.
 */
#pragma once
#include <oska/all.h>
#include <sdioemb/slot_api.h>
#include <sdioemb/windows/slot_api.h>
#include <sdioemb/slot_ushc.h>
#include <sdioemb/trace.h>

#include <usbdi.h>
#include <usbdlib.h>
#include <Wdfusb.h>

DRIVER_INITIALIZE DriverEntry;
EVT_WDF_OBJECT_CONTEXT_CLEANUP UshcDriverCleanup;
EVT_WDF_OBJECT_CONTEXT_CLEANUP UshcEvtDeviceCleanup;
EVT_WDF_DRIVER_DEVICE_ADD UshcEvtDeviceAdd;
EVT_WDF_DEVICE_PREPARE_HARDWARE UshcEvtDevicePrepareHardware;
EVT_WDF_DEVICE_RELEASE_HARDWARE UshcEvtDeviceReleaseHardware;
EVT_WDF_DEVICE_D0_ENTRY UshcEvtDeviceD0Entry;
EVT_WDF_DEVICE_D0_EXIT UshcEvtDeviceD0Exit;
EVT_WDF_USB_READER_COMPLETION_ROUTINE InterruptPipeReadComplete;
EVT_WDF_REQUEST_COMPLETION_ROUTINE CbwRequestCompletionRoutine;
EVT_WDF_REQUEST_COMPLETION_ROUTINE DataRequestCompletionRoutine;
EVT_WDF_REQUEST_COMPLETION_ROUTINE CswRequestCompletionRoutine;
EVT_WDF_USB_READERS_FAILED InterruptPipeReadFailed;
EVT_WDF_WORKITEM Worker;
EVT_WDF_WORKITEM UshcSetBusFreqWork;

typedef struct sdioemb_slot SDIOEMB_SLOT;
typedef struct sdioemb_slot* PSDIOEMB_SLOT;

typedef struct sdioemb_cmd SDIO_CMD;
typedef struct sdioemb_cmd* PSDIO_CMD;

typedef enum ushc_request USHC_REQUEST;
typedef enum ushc_request_direction USHC_REQUEST_DIRECTION; 
typedef enum ushc_request_recipient USHC_REQUEST_RECIPIENT;

typedef struct sdioemb_ushc SDIOEMB_USHC;
typedef enum sdioemb_power SDIOEMB_POWER;

#pragma pack (push, 1) 

typedef struct _CBW 
{
#define USHC_SDIO_CBW_SIGNATURE 'C'
    UCHAR Signature;
    UCHAR CmdIndex;
    INT16 BlockSize;
    UINT32 Arg;
}
CBW, *PCBW;

typedef struct _CSW 
{
#define USHC_SDIO_CSW_SIGNATURE 'S'
    UCHAR Signature;
    UCHAR Status;
    UINT32 Response;
}
CSW, *PSBW;

typedef struct _USHC_INT_DATA 
{
    UCHAR Status;
    UCHAR Reserved[3];
}
USHC_INT_DATA, *PUSHC_INT_DATA; 

#pragma pack (pop)


typedef struct _USHC_DEVICE_CONTEXT 
{
    WDFUSBDEVICE WdfUsbTargetDevice;
    USB_DEVICE_DESCRIPTOR UsbDeviceDescriptor;
    PUSB_CONFIGURATION_DESCRIPTOR UsbConfigurationDescriptor;
    WDFUSBINTERFACE UsbInterface;
    WDFUSBPIPE DataOutPipe;
    WDFUSBPIPE DataInPipe;
    WDFUSBPIPE InterruptPipe;

    UINT32 ControlerCapabilities;
    UINT16 ControlerState;
    INT32 NumSlots;
    SDIOEMB_SLOT **Slots;
    bool Registered;
} 
USHC_DEVICE_CONTEXT, *PUSHC_DEVICE_CONTEXT;
WDF_DECLARE_CONTEXT_TYPE_WITH_NAME(USHC_DEVICE_CONTEXT, UshcGetDeviceContext);

typedef struct _USHC_DATA 
{
    struct sdioemb_wdf_slot_data wdf_data;    // must be first 
    SDIOEMB_USHC Ushc;
    WDFDEVICE Device;
    WDFSPINLOCK Lock;
    CBW Cbw;
    WDFREQUEST CbwRequest;
    WDFMEMORY CbwMemory;
    CSW Csw;
    WDFREQUEST CswRequest;
    WDFMEMORY CswMemory;

    LONG CommandIoCount;

    bool SetBusFreqPending;
    int PendingBusFreq;
    struct sdioemb_cmd *PendingCmd;
    WDFWORKITEM SetBusFreqWork;
}
USHC_DATA, *PUSHC_DATA;

typedef struct _WORKER_CONTEXT 
{
    PUSHC_DATA Ushc;
    WDFUSBPIPE Pipe;
}
WORKER_CONTEXT, *PWORKER_CONTEXT;
WDF_DECLARE_CONTEXT_TYPE_WITH_NAME(WORKER_CONTEXT, GetWorkerContext)

NTSTATUS UshcRegistryInit(__in WDFDEVICE Device);

NTSTATUS UshcCreateSlot(__in WDFDEVICE Device, __out PSDIOEMB_SLOT *NewSlot, __in INT SlotIndex);
VOID UshcDestroySlot(__in WDFDEVICE Device, __in PSDIOEMB_SLOT Slot);

NTSTATUS SelectInterface(__in WDFDEVICE Device);
NTSTATUS SetControler(__in PUSHC_DEVICE_CONTEXT Dc, __in UINT16 ControlerState);
INT UshcSetBusFreq(__in PSDIOEMB_SLOT Slot, __in INT Clk);
INT UshcSetBusWidth(__in PSDIOEMB_SLOT Slot, __in INT BusWidth);
INT UshcSetHostControl(__in PSDIOEMB_SLOT Slot, __in UINT16 ControlerState);
INT UshcSubmitVendorRequest(__in PSDIOEMB_SLOT Slot, 
                            __in USHC_REQUEST Request,
                            __in USHC_REQUEST_DIRECTION Direction, 
                            __in USHC_REQUEST_RECIPIENT Recipient,
                            __in UINT16 Value, 
                            __in UINT16 Index, 
                            __in PVOID IoBuffer, 
                            __in UINT32 IoBufferLength);
INT UshcSubmitCbwRequest(__in PSDIOEMB_SLOT Slot, __in UCHAR CmdIndex, __in UINT16 BlockSize, __in UINT32 CmdArg);
INT UshcSubmitDataRequest(__in PSDIOEMB_SLOT Slot,
                          __in USHC_REQUEST_DIRECTION Direction,
                          __in_bcount(RequestBufferLength) PVOID RequestBuffer,
                          __in UINT32 RequestBufferLength);
INT UshcSubmitCswRequest(__in PSDIOEMB_SLOT Slot);
INT NTStatus2Posix(__in NTSTATUS Status);
NTSTATUS SubmitVendorRequest(__in WDFUSBDEVICE WdfUsbTargetDevice, 
                             __in UCHAR RequestCode, 
                             __in WDF_USB_BMREQUEST_DIRECTION RequestDirection, 
                             __in WDF_USB_BMREQUEST_RECIPIENT RequestRecipient,
                             __in USHORT Value,
                             __in USHORT Index,
                             __in_bcount_opt(TransferBufferLength) PVOID RequestBuffer,
                             __in ULONG RequestBufferLength);

NTSTATUS QPipeResetWorkItem(__in PUSHC_DATA Ushc, __in WDFUSBPIPE Pipe);
NTSTATUS ResetPipe(__in PUSHC_DATA Ushc, __in WDFUSBPIPE Pipe);
